local info, lib = ...

Library = Library or {}
Library.Flux = lib

lib.Addons = {}

function lib.Register( name, defaults )
	defaults = defaults or {}
	local addon = Inspect.Addon.Current()
	local info = Inspect.Addon.Detail( addon ).toc
	local data = Inspect.Addon.Detail( addon ).data
	local dbname = addon .. "_" .. name

	if not info.SavedVariables or not info.SavedVariables[dbname] then
		error( "tried to register database with invalid name \"" .. dbname .. "\", check your SavedVariables" )
	end

	local saved = info.SavedVariables[dbname]

	lib.Addons[addon] = lib.Addons[addon] or {}
	local db = setmetatable( {}, lib._DB )
	lib.Addons[addon][name] = db
	db:SetDefaults( defaults )
end

function lib.GetCharProfile()
	return Inspect.Unit.Detail("player").name .. "@" .. Inspect.Shard().name
end

function lib.OnSave( id )
	if not lib.Addons[id] then return end

	local dbs = lib.Addons[id]
	for k, v in pairs( dbs ) do
		v:GetTable().keys[lib.GetCharProfile()] = v:GetProfile()
		_G[id .. "_" .. k] = v:GetTable()
	end
end

function lib.OnLoad( id )
	if not lib.Addons[id] then return end

	if not Inspect.Unit.Detail("player") then
		lib.shouldload = true
		return
	end

	local addon = Inspect.Addon.Detail( id )
	local dbs = lib.Addons[id]
	for k, v in pairs( dbs ) do
		v:SetTable( _G[id .. "_" .. k] or {} )
		v:InitTable()
		addon.data[k] = v
	end
end

function lib.OnUnitChange( units )
	if not lib.shouldload then return end

	for k, v in pairs( units ) do
		if v == "player" then
			for id, dbs in pairs( lib.Addons ) do
				local addon = Inspect.Addon.Detail( id )
				for k, v in pairs( dbs ) do
					v:SetTable( _G[id .. "_" .. k] or {} )
					v:InitTable()
					addon.data[k] = v
				end
			end
			lib.shouldload = nil
		end
	end
end

table.insert( Event.Addon.SavedVariables.Save.Begin, { lib.OnSave, info.id, "save" } )
table.insert( Event.Addon.SavedVariables.Load.End, { lib.OnLoad, info.id, "load" } )
table.insert( Event.Unit.Available, { lib.OnUnitChange, info.id, "load2" } )
